# YASARA MOVIE
# TOPIC:    4. Molecular Modeling
# TITLE:    4.4. Domain replacement
# REQUIRES: Model
# AUTHOR:   Ahmet Sacan
# LICENSE:  GPL
# DESCRIPTION: A tutorial explaining how to replace a loop or an entire domain, for example to create a hybrid template for homology modeling. Many thanks to Sven Geier for his 'Hidden' background (more at www.sgeier.net/fractals).

# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
RequireVersion 8.7.4
OnError Continue

# Don't show console and head up display
Console Off
HUD Off
# Set default parameters
CoordSys Left
Projection Perspective
StickRadius 50
BallStickRadius 50,50
AtomTexture 1
Fog 10
ColorFog Black
AtomSize 100
SurfPar Resolution=4,Molecular=Gaussian
LightSource Alpha=45,Gamma=45,Ambience=70,Shadow=40

# Load background
Clear
LoadPNG bluegrass
LoadPNG hidden
ShowImage hidden

#Go DomainReplacement

#
# The Title
#
Title:
ClearSce
LoadSce title

# drop title and yami onto the grass
AutoPosObj title,Y=-29,Steps=50,Wait=No
AutoPosObj yami1,Y=0,Steps=50
# Yami jumps Off the title to the grass and comes closer to the user.
for i=1 to 3
  for j=0 to 180 step 3
    if i==2
      AutoMoveObj yami1,Y=(-13.8e0/61),Z=((125.-320)/122)
    elif i==3
      AutoMoveObj yami1,Y=((-6.7e0+13.8)/61)
    MoveObj yami1,Y=(cos j)
    if i==2 or i==3
      OriObj yami1,Alpha=((-j*2)%360)
    Wait 1
AutoMoveObj yami1,Y=0,Z=0

# Move title to the top
for j=0 to 450 step 5
  if j<=360
    PosObj title,Y=(-29.+sin (j%180)*10)
  else
    PosObj title,Y=(-29.+sin (j-360)*(85+29))
  Wait 1

# show second yami
yamipos()=PosObj yami1
PosObj yami2,(yamipos)
SwitchObj yami2,On

# split yamis
AutoRotateObj yami1,Y=-1
AutoRotateObj yami2,Y=1
Wait 70
AutoOriObj yami1,Beta=290,Steps=20,Wait=No
AutoOriObj yami2,Beta=70,Steps=20

# yamis jump to their own locations
yamix=yamipos1
yamiy=yamipos2
for i=1 to 2
  for j=0 to 180 step 3
    if i==1
      PosObj yami1 yami2,Y=(yamiy+sin j*20)
    else
      PosObj yami1,X=(yamix+(-50.-yamix)*j/180),Y=(yamiy+sin j*40)
      PosObj yami2,X=(yamix+(50.-yamix)*j/180),Y=(yamiy+sin j*40)
    if i==2
      OriObj yami1 yami2,Alpha=(j*2%360)
    Wait 1

ShowMessage 'Hi (FirstName)! Click the Continue-button to start the domain replacement tutorial...'
# Take snapshot here, Eliza!

# Exchange right legs continuously
ShowButton Continue
while !ContinueButton
  MarkAtom CA Res Ala 70 Obj yami1,CA Res Ala 70 Obj yami2
  Wait 60
  # yamis jump
  for i=1 to 2
    for j=0 to 180 step 3
      if i
        PosObj yami1 yami2,Y=(yamiy+sin j*20)
      if i==2
        OriObj yami1 yami2,Alpha=(j*2)
      Wait 1
  # split legs
  legn()=ListAtom N Res Ala 71 Obj yami1 yami2
  NameMol B or _,A
  JoinRes Obj yami1 yami2
  Wait 20
  if ContinueButton
    break
  SplitRes Ala 71 Obj yami1 yami2
  yami1()=SplitObj yami1
  yami2()=SplitObj yami2
  MarkAtom None
  # highlight title
  AutoPosObj title,Y=73,Z=277,Steps=9
  AutoPosObj title,Y=85,Z=320,Steps=11
  AutoPosObj title,Y=76,Z=290,Steps=9
  AutoPosObj title,Y=85,Z=320,Steps=18
  # Save leg locations for exchange
  leg1=(yami12)
  leg2=(yami22)
  legpos1()=PosObj (leg1)
  legpos2()=PosObj (leg2)
  legori1()=OriObj (leg1)
  legori2()=OriObj (leg2)
  # Legs will flip, meet in the middle, and exchange locations
  AutoPosObj (leg1) (leg2),X=0,Y=10,Steps=50,Wait=No
  AutoOriObj (leg2),180,280,0,Steps=50,Wait=No
  AutoOriObj (leg1),180,90,0,Steps=50
  AutoPosObj (leg1),(legpos2),Steps=50,Wait=No
  AutoPosObj (leg2),(legpos1),Steps=50,Wait=No
  AutoOriObj (leg1),(legori2),Steps=50,Wait=No
  AutoOriObj (leg2),(legori1),Steps=50
  # Join leg to the body
  JoinObj (leg2),(yami11)
  JoinObj (leg1),(yami21)
  NameObj (yami11),yami1
  NameObj (yami21),yami2
  JoinRes Obj yami1 yami2
  for i=1 to 2
    AddBond C Res Ala 70 Obj yami(i),N Res Ala 71 Obj yami(i),Update=No
HideMessage
# title sinks
AutoPosObj title,Y=-210,Z=450,Steps=100,Wait=No
AutoOriObj title,gamma=-90,Steps=100
SwitchObj title,Off
# yamis jump up
for j=0 to 270 step 5
  if j<=180
    PosObj yami1 yami2,Y=(yamiy+sin j*10)
  else
    PosObj yami1 yami2,Y=(yamiy+sin (j-180)*20)
    OriObj yami1,Gamma=-(j-180)
    OriObj yami2,Gamma=(j-180)
  Wait 1
# yamis fly away and disappear
AutoRotateObj yami1,Z=-5
AutoRotateObj yami2,Z=5
AutoPosObj yami1,0,177,1200,Steps=180,Wait=No
AutoPosObj yami2,0,177,1200,Steps=180,Wait=No
AutoOriObj grass,Alpha=120,Steps=180,Wait=No
AutoPosObj grass,Y=-99,Steps=180
SwitchObj yami1 yami2 grass,Off

#
# Intro for ATPase and the domain replacement task
# 
Intro:
ClearSce
LoadSce atpase
Switch Off
# zoom on cell and then atpase
LoadPNG cell
ShowImage cell,-5000,-4000,10000,10000,Alpha=0,Priority=1
AutoMoveImage cell,-20,-20,500,500,Alpha=100,Steps=200
Wait 201
LoadPNG atpase
ShowImage atpase,-5300,-1400,10000,10000,Alpha=20
AutoMoveImage cell,250,55,40,40,Alpha=30,Steps=200
AutoMoveImage atpase,160,300,640,480,Alpha=100,Steps=200
Wait 201
DelImage atpase
SwitchObj not A P N,on
AutoMoveImage cell,25,15,10,10,Alpha=0,Steps=50
#AutoMoveImage atpase,192,260,640,480,Alpha=0,Steps=50
Wait 55
DelImage cell
#DelImage atpase
ShowMessage 'P-type ATPases are ATP-powered ion pumps embedded in the cell membrane.'
Wait ContinueButton
ShowMessage "Calcium ATPase is such a protein responsible for transporting Ca++ ions, a representative PDB structure is shown here (2ZBD)."
LabelObj 2zbd,Format=2ZBD,Height=10.0,Color=Cyan,0,-55.0
Wait ContinueButton

# split away the membranes
AutoPosObj memb1,X=-220,Steps=200,Wait=No
AutoPosObj memb2,X=220,Steps=200
SwitchObj memb1 memb2,Off

# show the domains
secs='T','The transmembrane domain of the protein is composed of 10 transmembrane helices.','44-123 243-329 742-994','Cyan',
     'A',"The A(ctuator) domain regulates the transmembrane gate.",'1-43 124-242','yellow',
     'P',"The P(hosphorylation) domain contains the crucial Asp residue that accepts a phosphate from the ATP.",'330-360 601-741','Magenta',
     'N',"The N(ucleotide binding) domain contains the actual ATP binding site.",'361-601','red'
for i=1 to count secs step 4
  ShowMessage (secs(i+1))
  for j=1 to 5
    if j%2==1
      ColorRes (secs(i+2)) Obj 2zbd,(secs(i+3))
      if i!=1
        SwitchObj (secs(i)),On
    else
      ColorRes (secs(i+2)) Obj 2zbd,gray
      if i!=1
        SwitchObj (secs(i)),Off
    wait 5
  Wait ContinueButton

# Move 2zbd to the left
DelObj A P N
NumberObj All,1
AutoRotateObj 2zbd,Y=1
AutoPosObj 2zbd,-65,-4,Steps=50

# Label the N-domain
SegRes 361-601,NDom
LabelSeg NDom,Format=N,Height=10,Color=ff6060

# Introduce the problem
ShowMessage "Imagine you want to build a homology model of a related ATPase: ATP7B, a hepatic copper transporter."
MakeTextObj page,Width=110,Height=100
PosObj page,185,108,1000
PosText 0%,0%
Font Arial,Height=3,Color=Yellow,Depth=0
Print '>sp|P35670|ATP7B\nMPEQERQITAREGASRKILSKL\nSLPTRAWEPAMKKSFAFDNVGY\nEGGLDGLGPSSQVATSTVRILG\nMTCQSCVKSIEDRISNLKGIIS\nMKVSLEQGSATVKYVPSVVCLQ\nQVCHQIGDMGFEASIAEGKAAS\nWPSRSLPAQEAVVKLRVEGMTC\nQSCVSSIEGKVRKLQGVVRVKV\nSLSNQEAVITYQPYLIQPEDLR\nDHVNDMGFEAAIKSKVAPLSLG\nPIDIERLQSTNPKRPLSSANQN\nFNNSETLGHQGSHVVTLQLRID\n'
AutoPosObj page,45,79,117,Steps=50
Wait ContinueButton
ShowMessage "A BLAST search not only yields 2ZBD as the closest modeling template (in December 2008)..."
Wait ContinueButton
ShowMessage "...but also reveals that the PDB already contains at least the N-domain of ATP7B (PDB ID 2ARF, an NMR structure)."
OriObj 2arf,175,0,0
CenterObj 2arf
AutoRotateObj 2arf,Y=-1
ColorObj 2arf,Green
SwitchObj 2arf,On
AutoPosObj 2arf,38,-15,166,Steps=100,Wait=No
AutoPosObj page,38,76,-2,Steps=50,Wait=No
Wait 100
DelObj page
LabelObj 2arf,Format=N,Height=10,Color=60ff60
LabelObj 2arf,Format=2ARF,Height=7.0,Color=Green,0,-30.0
Wait ContinueButton
ShowMessage "Since the N-domain of Calcium ATPase (red) differs quite a bit from the one in ATP7B (green), it makes sense to replace it:"
Wait ContinueButton
ShowMessage "By replacing the red template N-domain with the real one (green), you create a hybrid template that is closer to the answer." 
Wait ContinueButton
AutoOriObj 2zbd 2arf,Beta=0,Steps=100
HideMessage
#SaveSce 2zbd2arf
ShowMessage 'The remainder of this tutorial will show you how this domain replacement can be performed.'
Wait ContinueButton

#
# DomainReplacement: This is the interactive part of the tutorial
#
DomainReplacement:
ClearSce
LoadSce 2zbd2arf
RemoveObj memb1 memb2
Style tube
objlist() = SplitObj 2zbd
NameObj (objlist1),2zbd
DelObj (objlist2) (objlist3)
NumberObj All,1
UserInput On

# Align structures
textcomlist()="The first task is to structurally align the two N-domains to find good joining points.",'',
              "Click Analyze > Align > Objects with MUSTANG to align the two structures - this may take a while (or just click Continue).",'SupAtom CA Res 1037-1044 1188-1196 Obj 2arf, CA Res 361-368 593-601 Obj 2zbd',
              "Now the N-domain of ATP7B has been structurally aligned with the corresponding N-domain in the modeling template.",'',
              "The next steps are most easily performed by showing Calpha atoms only, press <F4> to show a Calpha trace.",'Style Trace'
ShowTextCom textcomlist

# Mark joining residues
restomark='361 Obj 2zbd','Color=Red,X=0.5,Y=-1','1037 Obj 2arf','Color=Green,X=1.1,Y=1.1','601 Obj 2zbd','Color=Red,X=1.1,Y=1.1','1196 Obj 2arf','Color=Green,X=2.6,Y=0.5'
MarkAtom CA Res (restomark1),CA Res (restomark3),CA Res (restomark5),CA Res (restomark7)
for i=1 to count restomark step 2
  LabelRes (restomark(i)),RESName RESNUM,Height=0.4,(restomark(i+1))
ShowMessage "Now we need to replace the red domain with the green one. Good joining points are labeled."

# Zoom in on the joining points
AutoOriObj 2zbd 2arf,151.942,247.129,-32.073,Steps=100,Wait=No
AutoPosObj 2zbd 2arf,-22.164,8.801,17.684,Steps=100
for i=40 to -40 step -1
  HideRes 1-347 368-593 605-994 Obj 2zbd or 1044-1188 Obj 2arf and GlobalY>(i)
  Wait 1 

Wait ContinueButton
MarkAtom None
ShowMessage "There are two different ways to achieve that: a simple automatic one, and a manual step-by-step one, which is a good practice."

# Create empty image
MakeImage Buttons,Width=800,Height=600,TopCol=None
# Draw the buttons
Font Arial,Height=5%,Color=Black
ShowButton 'Click to show automatic method',X=400,Y=0,Width=800,Height=200,Border=40,
           Color=Green,Action='Go AutomaticReplacement'
ShowButton 'Click to show step-by-step method',X=400,Y=300,Width=800,Height=200,Border=40,
           Color=Red,Action='Go ManualReplacement'
# Show and animate the button image
ShowImage Buttons,X=-112,Y=-84,Width=1024,Height=768,Priority=1
AutoMoveImage Buttons,X=-54,Y=-42,Width=912,Height=684,Steps=50,Cycle=Yes
Wait forever

# 
# The automatic replacement method
#
AutomaticReplacement:
Wait 10
AutoMoveImage Buttons,X=380,Y=260,Width=40,Height=30,Steps=50,Cycle=off
Wait 50
DelImage Buttons
textcomlist()="The automatic method is as simple as selecting the residues to replace and their replacement, not only via Edit > Replace...",'',
              "...but also interactively from the atom context menu: Click on the first replacement residue (the green Pro 1037)...",'MarkAtom CA Res Pro 1037',
              "...then keep <Ctrl> pressed and click on three more residues: the last replacement (green Asp 1196)...",'MarkAtom CA Res Pro 1037,CA Res Asp 1196',
              "...then (<Ctrl> still pressed) the first replaced residue (red Met 361) and finally the last replaced residue (red Asp 601).",'MarkAtom CA Res Pro 1037,CA Res Asp 1196,CA Res Met 361,CA Res Asp 601',
              "Now right-click on the atom marked white (Pro 1037), select 'Replace', !uncheck! the option 'Superpose', and OK.",''
ShowTextCom textcomlist
if Atoms==10108
  # User didn't replace, do it now
  ReplaceRes Met 361-Asp 601 Obj 2zbd,Pro 1037-Asp 1196 Obj 2arf,Superpose=No
textcomlist()="At first glance, it looks like something went wrong, but that's just because the replacement is now present twice.",'HUD Obj',
              "Right-click on the replacement object 2arf in the HUD to open the object context menu and select 'Delete'.",'DelObj 2arf',
              "Note how YASARA automatically adjusted the peptide bonds and residue numbers during the replacement procedure.",'',
              "In practice, the next step would be to click Edit > Clean and perform an energy minimization to correct the geometry.",''
ShowTextCom textcomlist
Go ReplacementDone

# 
# The manual replacement method
#
ManualReplacement:
Wait 10
AutoMoveImage Buttons,X=380,Y=260,Width=40,Height=30,Steps=50,Cycle=off
Wait 50
DelImage Buttons

# Remove excess residues, split, and join
textcomlist()="Let's start by deleting the red N-domain in 2ZBD, which spans residues Met 361 to Asp 601...",'',
              "Click Edit > Delete > Residue, mark the residue range Met 361 to Asp 601 (using the Shift key), click OK.",'DelRes 361-601 Obj 2zbd',
              '','ColorRes 1032-1036 Obj 2arf,Blue',
              '','MarkAtom CA Res 1032 Obj 2arf',
              "Delete the blue surplus residues in 2ARF by clicking Delete > Residue in the atom context menu and repeating 4x with <Tab>.",'DelRes 1032-1036 Obj 2arf',
              "Next we split 2ZBD (magenta) into N- and C-terminal parts, so that the green 2ARF N-domain can be inserted in between.",'LabelRes 360 Obj 2zbd,RESName RESNUM,Height=0.4,color=Magenta,X=-0.8,Y=-0.7',
              '','LabelRes 602 Obj 2zbd,RESName RESNUM,Height=0.4,color=Magenta,X=1.9,Y=-0.6',
              '','MarkAtom CA Res 602 Obj 1',
              "Add a split point (i.e. a TER entry in PDB files): Right-click Pro 602, then Split > Residue.",'SplitRes Pro 602 Obj 2zbd',
              '','HUD Obj',
              "Now you can split object 2ZBD into child objects at the split points: Right-click object 2ZBD in the HUD, then click Split, OK.",'SplitObj 2zbd',
              '','MarkAtom CA Res Pro 1037,CA Res Gln 360',
              "Join the object marked white (2ARF) to the object marked red (N-terminal end of 2ZBD): Right-click Pro 1037, then Join > Object.",'JoinObj 2arf,1',
              'Indeed, you do not see much, but if you look in the object HUD, you can see that the 2ARF object has disappeared.','MarkAtom CA Res Pro 602,CA Res Asp 1196',
              "Now repeat in the other direction, joining the C-terminal end of 2ZBD to the rest: Right-click Pro 602, then Join > Object.",'JoinObj 2zbdA and not 1,1',
              '','MarkAtom None',
              'Remove all left-over split points: Click Edit > Join > Residue, double-click Obj 1.','JoinRes Obj 2zbdA',
              'And finally, renumber the residues: Click Edit > Number > Residue, double-click Obj 1, OK.','NumberRes Obj 2zbdA,1'
ShowTextCom textcomlist

# Add new peptide bonds
NameObj 2zbdA,2zbd
AutoOriObj 2zbd,152,247,-31,Steps=70,Wait=No
AutoPosObj 2zbd,-18,7,16,Steps=70
dstc=ListAtom C Res 360 Obj 2zbd
srcn=ListAtom N Res 361 Obj 2zbd
srcc=ListAtom C Res 520 Obj 2zbd
dstn=ListAtom N Res 521 Obj 2zbd
ShowAtom (dstc) (srcn)
LabelAtom (dstc),ATOMNAME,Height=0.4,color=Magenta,X=-0.5,Y=-0.5
LabelAtom (srcn),ATOMNAME,Height=0.4,color=Green,X=0,Y=0.5
textcomlist()="Now the only thing still missing are the peptide bonds (joining only combines objects, but does not add any bonds).",'MarkAtom (dstc),(srcn)',
              "Right-click on the C in Gln 360, click Add > Bond, make sure a single bond is selected, click OK.",'AddBond (dstc),(srcn),1.00,Update=No'
ShowTextCom textcomlist
UnlabelAtom All
HideAtom HA (dstc) (srcn)
MarkAtom None
AutoOriObj 2zbd,163,254,-9,Steps=70,Wait=No
AutoPosObj 2zbd,-17,15,13,Steps=70
srcc=ListAtom C Res 520 Obj 2zbd
dstn=ListAtom N Res 521 Obj 2zbd
ShowAtom (srcc) (dstn)
LabelAtom (srcc),ATOMNAME,Height=0.4,color=Green,X=0.4,Y=-0.2
LabelAtom (dstn),ATOMNAME,Height=0.4,color=Magenta,X=0,Y=0.5
MarkAtom (srcc),(dstn)
textcomlist()='And the second peptide bond, right-click on the C in Asp 520, click Add > Bond, OK.','AddBond (srcc),(dstn),1.00,Update=No'
ShowTextCom textcomlist
UserInput Off
HideAtom HA (srcc) (dstn)
MarkAtom None
HUD Off

# view the whole structure
ReplacementDone:
ShowMessage 'Congratulations, the hybrid template is now ready for homology modeling!'
Wait ContinueButton
Style Trace
AutoPosObj 2zbd,11,3,134,Steps=100,Wait=No
AutoOriObj 2zbd,165,242,-4,Steps=100
UnlabelRes All
AutoPosObj 2zbd,4.304,-3.351,217.612,Steps=100,Wait=No
AutoOriObj 2zbd,86.971,92.457,11.203,Steps=100
lasti=1
console Off
ShowMessage "You can of course use the very same domain replacement procedure to replace smaller fragments like loops."
for i=1 to 913 step 10
  HideRes (lasti)-(i) Obj 2zbd
  ShowSecRes (lasti)-(i) Obj 2zbd,Cartoon
  lasti=i+1
  Wait 1
HideRes (lasti)-913 Obj 2zbd
ShowSecRes (lasti)-913 Obj 2zbd,Cartoon
# bring the membranes back into view and rotate the new atpase
AddObj memb1 memb2
SwitchObj memb1 memb2,On
PosObj memb1,X=-220,Y=35,Z=212
PosObj memb2,X=220,Y=35,Z=212
OriObj memb?,0,0,0
AutoPosObj memb1,X=-59.666,Steps=200,Wait=No
AutoPosObj memb2,X=86.258,Steps=200
AutoRotate Y=0.4
UserInput On
ShowMessage "That's it for now. Click 'Continue' to end the tutorial."
Wait ContinueButton

# CLEAR THE SCENE
# ===============
# The background is left untouched.
def ClearSce
  DelAll
  DelImage not Hidden bluegrass
  HideMessage
  Userinput Off

# SHOW TEXT AND EXECUTE COMMAND
# =============================
def ShowTextCom textcomlist()
  for i=1 to count textcomlist step 2
    if textcomlist(i)==''
      HideMessage
    else
      ShowMessage (textcomlist(i))
      Wait ContinueButton
    if textcomlist(i+1)!=''
      (textcomlist(i+1))

# BUILD THE TITLE
# ===============
def BuildTitle text,pos
  title=BuildText Font=Arial,Height=14,Stroke=50%,Spacing=1.6,Element=Carbon,Bonds=Yes,Text=(text)
  HideObj (title)
  ColorObj (title),Yellow
  PosObj (title),(pos)
  ShowSurfObj (title),Molecular,dynamic,OutCol=c0c000,OutAlpha=100
  return title
